"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ansi_escapes_1 = __importDefault(require("ansi-escapes"));
const logging_1 = require("../logging");
const SHOW_ALTERNATE_SCREEN = '\u001B[?1049h';
const HIDE_ALTERNATE_SCREEN = '\u001B[?1049l';
const isWindows = process.platform === 'win32';
const isMintty = isWindows && !!(process.env.SHELL || process.env.TERM);
const isConEmuAnsiOn = (process.env.ConEmuANSI || '').toLowerCase() === 'on';
const supportsAnsi = !isWindows || isMintty || isConEmuAnsiOn;
function alternateScreen(enabled) {
    if (!enabled) {
        let needAnnounce = true;
        return {
            open() { },
            close() { },
            reset(heading) {
                if (needAnnounce) {
                    logging_1.stderr(heading);
                    needAnnounce = false;
                }
            }
        };
    }
    return {
        open() {
            if (supportsAnsi) {
                process.stderr.write(SHOW_ALTERNATE_SCREEN);
            }
        },
        close() {
            if (supportsAnsi) {
                process.stderr.write(HIDE_ALTERNATE_SCREEN);
            }
        },
        reset(heading) {
            logging_1.stderr(`${ansi_escapes_1.default.eraseScreen}${ansi_escapes_1.default.cursorTo(0, 0)}${heading}`);
        }
    };
}
exports.default = alternateScreen;
