"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const pretty_ms_1 = __importDefault(require("pretty-ms"));
const rollup = __importStar(require("rollup"));
const turbocolor_1 = __importDefault(require("turbocolor"));
const relativeId_1 = __importDefault(require("../../../src/utils/relativeId"));
const logging_1 = require("../logging");
const sourceMappingUrl_1 = __importDefault(require("../sourceMappingUrl"));
const timings_1 = require("./timings");
function build(inputOptions, outputOptions, warnings, silent = false) {
    const useStdout = !outputOptions[0].file && !outputOptions[0].dir;
    const start = Date.now();
    const files = useStdout ? ['stdout'] : outputOptions.map(t => relativeId_1.default(t.file || t.dir));
    if (!silent) {
        let inputFiles;
        if (typeof inputOptions.input === 'string') {
            inputFiles = inputOptions.input;
        }
        else if (inputOptions.input instanceof Array) {
            inputFiles = inputOptions.input.join(', ');
        }
        else if (typeof inputOptions.input === 'object' && inputOptions.input !== null) {
            inputFiles = Object.keys(inputOptions.input)
                .map(name => inputOptions.input[name])
                .join(', ');
        }
        logging_1.stderr(turbocolor_1.default.cyan(`\n${turbocolor_1.default.bold(inputFiles)} → ${turbocolor_1.default.bold(files.join(', '))}...`));
    }
    return rollup
        .rollup(inputOptions)
        .then((bundle) => {
        if (useStdout) {
            const output = outputOptions[0];
            if (output.sourcemap && output.sourcemap !== 'inline') {
                logging_1.handleError({
                    code: 'MISSING_OUTPUT_OPTION',
                    message: 'You must specify a --file (-o) option when creating a file with a sourcemap'
                });
            }
            return bundle.generate(output).then(({ output: outputs }) => {
                for (const file of outputs) {
                    let source;
                    if (file.isAsset) {
                        source = file.source;
                    }
                    else {
                        source = file.code;
                        if (output.sourcemap === 'inline') {
                            source += `\n//# ${sourceMappingUrl_1.default}=${file.map.toUrl()}\n`;
                        }
                    }
                    if (outputs.length > 1)
                        process.stdout.write('\n' + turbocolor_1.default.cyan(turbocolor_1.default.bold('//→ ' + file.fileName + ':')) + '\n');
                    process.stdout.write(source);
                }
            });
        }
        return Promise.all(outputOptions.map(output => bundle.write(output))).then(() => bundle);
    })
        .then((bundle) => {
        warnings.flush();
        if (!silent)
            logging_1.stderr(turbocolor_1.default.green(`created ${turbocolor_1.default.bold(files.join(', '))} in ${turbocolor_1.default.bold(pretty_ms_1.default(Date.now() - start))}`));
        if (bundle && bundle.getTimings) {
            timings_1.printTimings(bundle.getTimings());
        }
    })
        .catch((err) => {
        if (warnings.count > 0)
            warnings.flush();
        logging_1.handleError(err);
    });
}
exports.default = build;
