"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const require_relative_1 = __importDefault(require("require-relative"));
const mergeOptions_1 = __importDefault(require("../../../src/utils/mergeOptions"));
const relativeId_1 = require("../../../src/utils/relativeId");
const logging_1 = require("../logging");
const batchWarnings_1 = __importDefault(require("./batchWarnings"));
const build_1 = __importDefault(require("./build"));
const loadConfigFile_1 = __importDefault(require("./loadConfigFile"));
const watch_1 = __importDefault(require("./watch"));
function runRollup(command) {
    let inputSource;
    if (command._.length > 0) {
        if (command.input) {
            logging_1.handleError({
                code: 'DUPLICATE_IMPORT_OPTIONS',
                message: 'use --input, or pass input path as argument'
            });
        }
        inputSource = command._;
    }
    else if (typeof command.input === 'string') {
        inputSource = [command.input];
    }
    else {
        inputSource = command.input;
    }
    if (inputSource && inputSource.length > 0) {
        if (inputSource.some((input) => input.indexOf('=') !== -1)) {
            command.input = {};
            inputSource.forEach((input) => {
                const equalsIndex = input.indexOf('=');
                const value = input.substr(equalsIndex + 1);
                let key = input.substr(0, equalsIndex);
                if (!key)
                    key = relativeId_1.getAliasName(input);
                command.input[key] = value;
            });
        }
        else {
            command.input = inputSource;
        }
    }
    if (command.environment) {
        const environment = Array.isArray(command.environment)
            ? command.environment
            : [command.environment];
        environment.forEach((arg) => {
            arg.split(',').forEach((pair) => {
                const [key, value] = pair.split(':');
                if (value) {
                    process.env[key] = value;
                }
                else {
                    process.env[key] = String(true);
                }
            });
        });
    }
    let configFile = command.config === true ? 'rollup.config.js' : command.config;
    if (configFile) {
        if (configFile.slice(0, 5) === 'node:') {
            const pkgName = configFile.slice(5);
            try {
                configFile = require_relative_1.default.resolve(`rollup-config-${pkgName}`, process.cwd());
            }
            catch (err) {
                try {
                    configFile = require_relative_1.default.resolve(pkgName, process.cwd());
                }
                catch (err) {
                    if (err.code === 'MODULE_NOT_FOUND') {
                        logging_1.handleError({
                            code: 'MISSING_EXTERNAL_CONFIG',
                            message: `Could not resolve config file ${configFile}`
                        });
                    }
                    throw err;
                }
            }
        }
        else {
            // find real path of config so it matches what Node provides to callbacks in require.extensions
            configFile = fs_1.realpathSync(configFile);
        }
        if (command.watch)
            process.env.ROLLUP_WATCH = 'true';
        loadConfigFile_1.default(configFile, command)
            .then(configs => execute(configFile, configs, command))
            .catch(logging_1.handleError);
    }
    else {
        return execute(configFile, [{ input: null }], command);
    }
}
exports.default = runRollup;
function execute(configFile, configs, command) {
    if (command.watch) {
        watch_1.default(configFile, configs, command, command.silent);
    }
    else {
        let promise = Promise.resolve();
        for (const config of configs) {
            promise = promise.then(() => {
                const warnings = batchWarnings_1.default();
                const { inputOptions, outputOptions, optionError } = mergeOptions_1.default({
                    command,
                    config,
                    defaultOnWarnHandler: warnings.add
                });
                if (optionError)
                    inputOptions.onwarn({ code: 'UNKNOWN_OPTION', message: optionError });
                return build_1.default(inputOptions, outputOptions, warnings, command.silent);
            });
        }
        return promise;
    }
}
