"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ReturnValueScope_1 = __importDefault(require("../scopes/ReturnValueScope"));
const values_1 = require("../values");
const BlockStatement_1 = __importDefault(require("./BlockStatement"));
const NodeType = __importStar(require("./NodeType"));
const Node_1 = require("./shared/Node");
class ArrowFunctionExpression extends Node_1.NodeBase {
    createScope(parentScope) {
        this.scope = new ReturnValueScope_1.default(parentScope, this.context);
    }
    deoptimizePath(path) {
        // A reassignment of UNKNOWN_PATH is considered equivalent to having lost track
        // which means the return expression needs to be reassigned
        if (path.length === 1 && path[0] === values_1.UNKNOWN_KEY) {
            this.scope.getReturnExpression().deoptimizePath(values_1.UNKNOWN_PATH);
        }
    }
    getReturnExpressionWhenCalledAtPath(path) {
        return path.length === 0 ? this.scope.getReturnExpression() : values_1.UNKNOWN_EXPRESSION;
    }
    hasEffects(_options) {
        return false;
    }
    hasEffectsWhenAccessedAtPath(path, _options) {
        return path.length > 1;
    }
    hasEffectsWhenAssignedAtPath(path, _options) {
        return path.length > 1;
    }
    hasEffectsWhenCalledAtPath(path, _callOptions, options) {
        if (path.length > 0) {
            return true;
        }
        for (const param of this.params) {
            if (param.hasEffects(options))
                return true;
        }
        return this.body.hasEffects(options);
    }
    initialise() {
        this.included = false;
        for (const param of this.params) {
            param.declare('parameter', values_1.UNKNOWN_EXPRESSION);
        }
        if (this.body instanceof BlockStatement_1.default) {
            this.body.addImplicitReturnExpressionToScope();
        }
        else {
            this.scope.addReturnExpression(this.body);
        }
    }
    parseNode(esTreeNode) {
        if (esTreeNode.body.type === NodeType.BlockStatement) {
            this.body = new this.context.nodeConstructors.BlockStatement(esTreeNode.body, this, this.scope.hoistedBodyVarScope);
        }
        super.parseNode(esTreeNode);
    }
}
exports.default = ArrowFunctionExpression;
ArrowFunctionExpression.prototype.preventChildBlockScope = true;
