"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const values_1 = require("../values");
const Node_1 = require("./shared/Node");
const binaryOperators = {
    '!=': (left, right) => left != right,
    '!==': (left, right) => left !== right,
    '%': (left, right) => left % right,
    '&': (left, right) => left & right,
    '*': (left, right) => left * right,
    // At the moment, "**" will be transpiled to Math.pow
    '**': (left, right) => Math.pow(left, right),
    '+': (left, right) => left + right,
    '-': (left, right) => left - right,
    '/': (left, right) => left / right,
    '<': (left, right) => left < right,
    '<<': (left, right) => left << right,
    '<=': (left, right) => left <= right,
    '==': (left, right) => left == right,
    '===': (left, right) => left === right,
    '>': (left, right) => left > right,
    '>=': (left, right) => left >= right,
    '>>': (left, right) => left >> right,
    '>>>': (left, right) => left >>> right,
    '^': (left, right) => left ^ right,
    in: () => values_1.UNKNOWN_VALUE,
    instanceof: () => values_1.UNKNOWN_VALUE,
    '|': (left, right) => left | right
};
class BinaryExpression extends Node_1.NodeBase {
    getLiteralValueAtPath(path, recursionTracker, origin) {
        if (path.length > 0)
            return values_1.UNKNOWN_VALUE;
        const leftValue = this.left.getLiteralValueAtPath(values_1.EMPTY_PATH, recursionTracker, origin);
        if (leftValue === values_1.UNKNOWN_VALUE)
            return values_1.UNKNOWN_VALUE;
        const rightValue = this.right.getLiteralValueAtPath(values_1.EMPTY_PATH, recursionTracker, origin);
        if (rightValue === values_1.UNKNOWN_VALUE)
            return values_1.UNKNOWN_VALUE;
        const operatorFn = binaryOperators[this.operator];
        if (!operatorFn)
            return values_1.UNKNOWN_VALUE;
        return operatorFn(leftValue, rightValue);
    }
    hasEffectsWhenAccessedAtPath(path, _options) {
        return path.length > 1;
    }
}
exports.default = BinaryExpression;
