"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const renderHelpers_1 = require("../../utils/renderHelpers");
const BlockScope_1 = __importDefault(require("../scopes/BlockScope"));
const values_1 = require("../values");
const NodeType = __importStar(require("./NodeType"));
const Node_1 = require("./shared/Node");
function isBlockStatement(node) {
    return node.type === NodeType.BlockStatement;
}
exports.isBlockStatement = isBlockStatement;
class BlockStatement extends Node_1.StatementBase {
    addImplicitReturnExpressionToScope() {
        const lastStatement = this.body[this.body.length - 1];
        if (!lastStatement || lastStatement.type !== NodeType.ReturnStatement) {
            this.scope.addReturnExpression(values_1.UNKNOWN_EXPRESSION);
        }
    }
    createScope(parentScope) {
        this.scope = this.parent.preventChildBlockScope
            ? parentScope
            : new BlockScope_1.default(parentScope);
    }
    hasEffects(options) {
        for (const node of this.body) {
            if (node.hasEffects(options))
                return true;
        }
    }
    include(includeAllChildrenRecursively) {
        this.included = true;
        for (const node of this.body) {
            if (includeAllChildrenRecursively || node.shouldBeIncluded())
                node.include(includeAllChildrenRecursively);
        }
    }
    render(code, options) {
        if (this.body.length) {
            renderHelpers_1.renderStatementList(this.body, code, this.start + 1, this.end - 1, options);
        }
        else {
            super.render(code, options);
        }
    }
}
exports.default = BlockStatement;
