"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const renderHelpers_1 = require("../../utils/renderHelpers");
const Node_1 = require("./shared/Node");
const getDynamicImportMechanism = (options) => {
    switch (options.format) {
        case 'cjs': {
            const _ = options.compact ? '' : ' ';
            return {
                interopLeft: `Promise.resolve({${_}default:${_}require(`,
                interopRight: `)${_}})`,
                left: 'Promise.resolve(require(',
                right: '))'
            };
        }
        case 'amd': {
            const _ = options.compact ? '' : ' ';
            const resolve = options.compact ? 'c' : 'resolve';
            const reject = options.compact ? 'e' : 'reject';
            return {
                interopLeft: `new Promise(function${_}(${resolve},${_}${reject})${_}{${_}require([`,
                interopRight: `],${_}function${_}(m)${_}{${_}${resolve}({${_}default:${_}m${_}})${_}},${_}${reject})${_}})`,
                left: `new Promise(function${_}(${resolve},${_}${reject})${_}{${_}require([`,
                right: `],${_}${resolve},${_}${reject})${_}})`
            };
        }
        case 'system':
            return {
                left: 'module.import(',
                right: ')'
            };
        case 'es':
            return {
                left: `${options.dynamicImportFunction || 'import'}(`,
                right: ')'
            };
    }
};
class Import extends Node_1.NodeBase {
    include() {
        this.included = true;
        this.context.includeDynamicImport(this);
    }
    initialise() {
        this.included = false;
        this.resolutionNamespace = undefined;
        this.resolutionInterop = false;
        this.context.addDynamicImport(this);
    }
    render(code, options) {
        if (this.resolutionNamespace) {
            const _ = options.compact ? '' : ' ';
            const s = options.compact ? '' : ';';
            code.overwrite(this.parent.start, this.parent.end, `Promise.resolve().then(function${_}()${_}{${_}return ${this.resolutionNamespace}${s}${_}})`);
            return;
        }
        const importMechanism = getDynamicImportMechanism(options);
        if (importMechanism) {
            const leftMechanism = (this.resolutionInterop && importMechanism.interopLeft) || importMechanism.left;
            const leftMechanismEnd = renderHelpers_1.findFirstOccurrenceOutsideComment(code.original, '(', this.parent.callee.end) + 1;
            code.overwrite(this.parent.start, leftMechanismEnd, leftMechanism);
            const rightMechanism = (this.resolutionInterop && importMechanism.interopRight) || importMechanism.right;
            code.overwrite(this.parent.end - 1, this.parent.end, rightMechanism);
        }
    }
    renderFinalResolution(code, resolution, format) {
        if (this.included) {
            if (format === 'amd' && resolution.startsWith("'.") && resolution.endsWith(".js'")) {
                resolution = resolution.slice(0, -4) + "'";
            }
            code.overwrite(this.parent.arguments[0].start, this.parent.arguments[0].end, resolution);
        }
    }
    setResolution(interop, namespace = undefined) {
        this.resolutionInterop = interop;
        this.resolutionNamespace = namespace;
    }
}
exports.default = Import;
