"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("../../utils/path");
const MemberExpression_1 = __importDefault(require("./MemberExpression"));
const Node_1 = require("./shared/Node");
const ASSET_PREFIX = 'ROLLUP_ASSET_URL_';
const CHUNK_PREFIX = 'ROLLUP_CHUNK_URL_';
class MetaProperty extends Node_1.NodeBase {
    hasEffectsWhenAccessedAtPath(path) {
        return path.length > 1;
    }
    initialise() {
        if (this.meta.name === 'import') {
            this.context.addImportMeta(this);
        }
        this.included = false;
    }
    renderFinalMechanism(code, chunkId, format, pluginDriver) {
        if (!this.included)
            return false;
        const parent = this.parent;
        const importMetaProperty = parent instanceof MemberExpression_1.default && typeof parent.propertyKey === 'string'
            ? parent.propertyKey
            : null;
        if (importMetaProperty &&
            (importMetaProperty.startsWith(ASSET_PREFIX) || importMetaProperty.startsWith(CHUNK_PREFIX))) {
            let assetReferenceId = null;
            let chunkReferenceId = null;
            let fileName;
            if (importMetaProperty.startsWith(ASSET_PREFIX)) {
                assetReferenceId = importMetaProperty.substr(ASSET_PREFIX.length);
                fileName = this.context.getAssetFileName(assetReferenceId);
            }
            else {
                chunkReferenceId = importMetaProperty.substr(CHUNK_PREFIX.length);
                fileName = this.context.getChunkFileName(chunkReferenceId);
            }
            const relativePath = path_1.normalize(path_1.relative(path_1.dirname(chunkId), fileName));
            let replacement;
            if (assetReferenceId !== null) {
                // deprecated hook for assets
                replacement = pluginDriver.hookFirstSync('resolveAssetUrl', [
                    {
                        assetFileName: fileName,
                        chunkId,
                        format,
                        moduleId: this.context.module.id,
                        relativeAssetPath: relativePath
                    }
                ]);
            }
            if (!replacement) {
                replacement = pluginDriver.hookFirstSync('resolveFileUrl', [
                    {
                        assetReferenceId,
                        chunkId,
                        chunkReferenceId,
                        fileName,
                        format,
                        moduleId: this.context.module.id,
                        relativePath
                    }
                ]);
            }
            code.overwrite(parent.start, parent.end, replacement);
            return true;
        }
        const replacement = pluginDriver.hookFirstSync('resolveImportMeta', [
            importMetaProperty,
            {
                chunkId,
                format,
                moduleId: this.context.module.id
            }
        ]);
        if (typeof replacement === 'string') {
            if (parent instanceof MemberExpression_1.default) {
                code.overwrite(parent.start, parent.end, replacement);
            }
            else {
                code.overwrite(this.start, this.end, replacement);
            }
            return true;
        }
        return false;
    }
}
exports.default = MetaProperty;
