"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const values_1 = require("../values");
const Node_1 = require("./shared/Node");
const unaryOperators = {
    '!': value => !value,
    '+': value => +value,
    '-': value => -value,
    delete: () => values_1.UNKNOWN_VALUE,
    typeof: value => typeof value,
    void: () => undefined,
    '~': value => ~value
};
class UnaryExpression extends Node_1.NodeBase {
    bind() {
        super.bind();
        if (this.operator === 'delete') {
            this.argument.deoptimizePath(values_1.EMPTY_PATH);
        }
    }
    getLiteralValueAtPath(path, recursionTracker, origin) {
        if (path.length > 0)
            return values_1.UNKNOWN_VALUE;
        const argumentValue = this.argument.getLiteralValueAtPath(values_1.EMPTY_PATH, recursionTracker, origin);
        if (argumentValue === values_1.UNKNOWN_VALUE)
            return values_1.UNKNOWN_VALUE;
        return unaryOperators[this.operator](argumentValue);
    }
    hasEffects(options) {
        return (this.argument.hasEffects(options) ||
            (this.operator === 'delete' &&
                this.argument.hasEffectsWhenAssignedAtPath(values_1.EMPTY_PATH, options)));
    }
    hasEffectsWhenAccessedAtPath(path, _options) {
        if (this.operator === 'void') {
            return path.length > 0;
        }
        return path.length > 1;
    }
}
exports.default = UnaryExpression;
