"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const values_1 = require("../values");
const Identifier_1 = require("./Identifier");
const Node_1 = require("./shared/Node");
class UpdateExpression extends Node_1.NodeBase {
    bind() {
        super.bind();
        this.argument.deoptimizePath(values_1.EMPTY_PATH);
        if (Identifier_1.isIdentifier(this.argument)) {
            const variable = this.scope.findVariable(this.argument.name);
            variable.isReassigned = true;
        }
    }
    hasEffects(options) {
        return (this.argument.hasEffects(options) ||
            this.argument.hasEffectsWhenAssignedAtPath(values_1.EMPTY_PATH, options));
    }
    hasEffectsWhenAccessedAtPath(path, _options) {
        return path.length > 1;
    }
    render(code, options) {
        this.argument.render(code, options);
        const variable = this.argument.variable;
        if (options.format === 'system' && variable && variable.exportName) {
            const name = variable.getName();
            if (this.prefix) {
                code.overwrite(this.start, this.end, `exports('${variable.exportName}', ${this.operator}${name})`);
            }
            else {
                let op;
                switch (this.operator) {
                    case '++':
                        op = `${name} + 1`;
                        break;
                    case '--':
                        op = `${name} - 1`;
                        break;
                }
                code.overwrite(this.start, this.end, `(exports('${variable.exportName}', ${op}), ${name}${this.operator})`);
            }
        }
    }
}
exports.default = UpdateExpression;
