"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const safeName_1 = require("../../utils/safeName");
const Scope_1 = __importDefault(require("./Scope"));
class ChildScope extends Scope_1.default {
    constructor(parent) {
        super();
        this.accessedOutsideVariables = Object.create(null);
        this.parent = parent;
        parent.children.push(this);
    }
    addNamespaceMemberAccess(name, variable) {
        this.accessedOutsideVariables[name] = variable;
        if (this.parent instanceof ChildScope) {
            this.parent.addNamespaceMemberAccess(name, variable);
        }
    }
    addReturnExpression(expression) {
        this.parent instanceof ChildScope && this.parent.addReturnExpression(expression);
    }
    contains(name) {
        return name in this.variables || this.parent.contains(name);
    }
    deconflict(forbiddenNames) {
        const usedNames = Object.assign(Object.create(null), forbiddenNames);
        for (const name of Object.keys(this.accessedOutsideVariables)) {
            const variable = this.accessedOutsideVariables[name];
            if (variable.included) {
                usedNames[variable.getBaseVariableName()] = true;
            }
        }
        for (const name of Object.keys(this.variables)) {
            const variable = this.variables[name];
            if (variable.included) {
                variable.setSafeName(safeName_1.getSafeName(name, usedNames));
            }
        }
        for (const scope of this.children) {
            scope.deconflict(forbiddenNames);
        }
    }
    findLexicalBoundary() {
        return this.parent instanceof ChildScope ? this.parent.findLexicalBoundary() : this;
    }
    findVariable(name) {
        const knownVariable = this.variables[name] || this.accessedOutsideVariables[name];
        if (knownVariable) {
            return knownVariable;
        }
        return (this.accessedOutsideVariables[name] = this.parent.findVariable(name));
    }
}
exports.default = ChildScope;
