"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const NodeType = __importStar(require("../nodes/NodeType"));
const LocalVariable_1 = __importDefault(require("./LocalVariable"));
function isExportDefaultVariable(variable) {
    return variable.isDefault;
}
exports.isExportDefaultVariable = isExportDefaultVariable;
class ExportDefaultVariable extends LocalVariable_1.default {
    constructor(name, exportDefaultDeclaration, context) {
        super(name, exportDefaultDeclaration, exportDefaultDeclaration.declaration, context);
        // Not initialised during construction
        this.originalId = null;
        const declaration = exportDefaultDeclaration.declaration;
        if ((declaration.type === NodeType.FunctionDeclaration ||
            declaration.type === NodeType.ClassDeclaration) &&
            declaration.id) {
            this.hasId = true;
            this.originalId = declaration.id;
        }
        else if (declaration.type === NodeType.Identifier) {
            this.originalId = declaration;
        }
    }
    addReference(identifier) {
        if (!this.hasId) {
            this.name = identifier.name;
        }
    }
    getName() {
        return this.referencesOriginal() ? this.originalId.variable.getName() : super.getName();
    }
    getOriginalVariable() {
        return (this.originalId && this.originalId.variable) || null;
    }
    getOriginalVariableName() {
        return (this.originalId && this.originalId.name) || null;
    }
    referencesOriginal() {
        return this.originalId && (this.hasId || !this.originalId.variable.isReassigned);
    }
    setRenderNames(baseName, name) {
        if (this.referencesOriginal()) {
            this.originalId.variable.setRenderNames(baseName, name);
        }
        else {
            super.setRenderNames(baseName, name);
        }
    }
    setSafeName(name) {
        if (this.referencesOriginal()) {
            this.originalId.variable.setSafeName(name);
        }
        else {
            super.setSafeName(name);
        }
    }
}
exports.default = ExportDefaultVariable;
ExportDefaultVariable.prototype.getBaseVariableName = ExportDefaultVariable.prototype.getName;
ExportDefaultVariable.prototype.isDefault = true;
