"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _256_1 = __importDefault(require("hash.js/lib/hash/sha/256"));
const error_1 = require("./error");
const path_1 = require("./path");
const referenceIds_1 = require("./referenceIds");
const relativeId_1 = require("./relativeId");
const renderNamePattern_1 = require("./renderNamePattern");
function getAssetFileName(asset, existingNames, assetFileNames) {
    if (asset.source === undefined)
        error_1.error(error_1.errNoAssetSourceSet(asset));
    if (asset.fileName)
        return asset.fileName;
    return renderNamePattern_1.makeUnique(renderNamePattern_1.renderNamePattern(assetFileNames, 'assetFileNames', name => {
        switch (name) {
            case 'hash':
                const hash = _256_1.default();
                hash.update(name);
                hash.update(':');
                hash.update(asset.source);
                return hash.digest('hex').substr(0, 8);
            case 'name':
                return asset.name.substr(0, asset.name.length - path_1.extname(asset.name).length);
            case 'extname':
                return path_1.extname(asset.name);
            case 'ext':
                return path_1.extname(asset.name).substr(1);
        }
    }), existingNames);
}
exports.getAssetFileName = getAssetFileName;
function createAssetPluginHooks(assetsByReferenceId, outputBundle, assetFileNames) {
    return {
        emitAsset(name, source) {
            if (typeof name !== 'string' || !relativeId_1.isPlainName(name))
                error_1.error(error_1.errInvalidAssetName(name));
            const asset = { name, source, fileName: undefined };
            if (outputBundle && source !== undefined)
                finaliseAsset(asset, outputBundle, assetFileNames);
            return referenceIds_1.addWithNewReferenceId(asset, assetsByReferenceId, name);
        },
        setAssetSource(assetReferenceId, source) {
            const asset = assetsByReferenceId.get(assetReferenceId);
            if (!asset)
                error_1.error(error_1.errAssetReferenceIdNotFoundForSetSource(assetReferenceId));
            if (asset.source !== undefined)
                error_1.error(error_1.errAssetSourceAlreadySet(asset));
            if (typeof source !== 'string' && !source)
                error_1.error(error_1.errAssetSourceMissingForSetSource(asset));
            asset.source = source;
            if (outputBundle)
                finaliseAsset(asset, outputBundle, assetFileNames);
        },
        getAssetFileName(assetReferenceId) {
            const asset = assetsByReferenceId.get(assetReferenceId);
            if (!asset)
                error_1.error(error_1.errAssetReferenceIdNotFoundForFilename(assetReferenceId));
            if (asset.fileName === undefined)
                error_1.error(error_1.errAssetNotFinalisedForFileName(asset));
            return asset.fileName;
        }
    };
}
exports.createAssetPluginHooks = createAssetPluginHooks;
function finaliseAsset(asset, outputBundle, assetFileNames) {
    const fileName = getAssetFileName(asset, outputBundle, assetFileNames);
    asset.fileName = fileName;
    outputBundle[fileName] = {
        fileName,
        isAsset: true,
        source: asset.source
    };
}
exports.finaliseAsset = finaliseAsset;
function createTransformEmitAsset(assetsByReferenceId, emitAsset) {
    const assets = [];
    return {
        assets,
        emitAsset: (name, source) => {
            const assetReferenceId = emitAsset(name, source);
            const asset = assetsByReferenceId.get(assetReferenceId);
            assets.push({
                fileName: undefined,
                name: asset.name,
                source: asset.source
            });
            return assetReferenceId;
        }
    };
}
exports.createTransformEmitAsset = createTransformEmitAsset;
