"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("./path");
function assignChunkIds(chunks, inputOptions, outputOptions, inputBase, addons) {
    const usedIds = {};
    const [entryChunks, otherChunks] = chunks.reduce(([entryChunks, otherChunks], chunk) => {
        (chunk.facadeModule && chunk.facadeModule.isUserDefinedEntryPoint
            ? entryChunks
            : otherChunks).push(chunk);
        return [entryChunks, otherChunks];
    }, [[], []]);
    // make sure entry chunk names take precedence with regard to deconflicting
    const chunksForNaming = entryChunks.concat(otherChunks);
    for (let i = 0; i < chunksForNaming.length; i++) {
        const chunk = chunksForNaming[i];
        if (outputOptions.file) {
            chunk.id = path_1.basename(outputOptions.file);
        }
        else if (inputOptions.preserveModules) {
            chunk.generateIdPreserveModules(inputBase, usedIds);
        }
        else {
            let pattern, patternName;
            if (chunk.facadeModule && chunk.facadeModule.isUserDefinedEntryPoint) {
                pattern = outputOptions.entryFileNames || '[name].js';
                patternName = 'output.entryFileNames';
            }
            else {
                pattern = outputOptions.chunkFileNames || '[name]-[hash].js';
                patternName = 'output.chunkFileNames';
            }
            chunk.generateId(pattern, patternName, addons, outputOptions, usedIds);
        }
        usedIds[chunk.id] = true;
    }
}
exports.assignChunkIds = assignChunkIds;
