"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const locate_character_1 = require("locate-character");
const getCodeFrame_1 = __importDefault(require("./getCodeFrame"));
const relativeId_1 = __importDefault(require("./relativeId"));
function error(base, props) {
    if (base instanceof Error === false)
        base = Object.assign(new Error(base.message), base);
    if (props)
        Object.assign(base, props);
    throw base;
}
exports.error = error;
function augmentCodeLocation(object, pos, source, id) {
    if (pos.line !== undefined && pos.column !== undefined) {
        const { line, column } = pos;
        object.loc = { file: id, line, column };
    }
    else {
        object.pos = pos;
        const { line, column } = locate_character_1.locate(source, pos, { offsetLine: 1 });
        object.loc = { file: id, line, column };
    }
    if (object.frame === undefined) {
        const { line, column } = object.loc;
        object.frame = getCodeFrame_1.default(source, line, column);
    }
}
exports.augmentCodeLocation = augmentCodeLocation;
var Errors;
(function (Errors) {
    Errors["ASSET_NOT_FINALISED"] = "ASSET_NOT_FINALISED";
    Errors["ASSET_NOT_FOUND"] = "ASSET_NOT_FOUND";
    Errors["ASSET_SOURCE_ALREADY_SET"] = "ASSET_SOURCE_ALREADY_SET";
    Errors["ASSET_SOURCE_MISSING"] = "ASSET_SOURCE_MISSING";
    Errors["BAD_LOADER"] = "BAD_LOADER";
    Errors["CHUNK_NOT_FOUND"] = "CHUNK_NOT_FOUND";
    Errors["CHUNK_NOT_GENERATED"] = "CHUNK_NOT_GENERATED";
    Errors["INVALID_ASSET_NAME"] = "INVALID_ASSET_NAME";
    Errors["INVALID_CHUNK"] = "INVALID_CHUNK";
    Errors["INVALID_EXTERNAL_ID"] = "INVALID_EXTERNAL_ID";
    Errors["INVALID_OPTION"] = "INVALID_OPTION";
    Errors["INVALID_PLUGIN_HOOK"] = "INVALID_PLUGIN_HOOK";
    Errors["INVALID_ROLLUP_PHASE"] = "INVALID_ROLLUP_PHASE";
    Errors["NAMESPACE_CONFLICT"] = "NAMESPACE_CONFLICT";
    Errors["UNRESOLVED_ENTRY"] = "UNRESOLVED_ENTRY";
    Errors["UNRESOLVED_IMPORT"] = "UNRESOLVED_IMPORT";
})(Errors = exports.Errors || (exports.Errors = {}));
function errAssetNotFinalisedForFileName(asset) {
    return {
        code: Errors.ASSET_NOT_FINALISED,
        message: `Plugin error - Unable to get file name for asset "${asset.name}". Ensure that the source is set and that generate is called first.`
    };
}
exports.errAssetNotFinalisedForFileName = errAssetNotFinalisedForFileName;
function errChunkNotGeneratedForFileName(entry) {
    return {
        code: Errors.CHUNK_NOT_GENERATED,
        message: `Plugin error - Unable to get file name for chunk "${entry.name}". Ensure that generate is called first.`
    };
}
exports.errChunkNotGeneratedForFileName = errChunkNotGeneratedForFileName;
function errAssetReferenceIdNotFoundForFilename(assetReferenceId) {
    return {
        code: Errors.ASSET_NOT_FOUND,
        message: `Plugin error - Unable to get file name for unknown asset "${assetReferenceId}".`
    };
}
exports.errAssetReferenceIdNotFoundForFilename = errAssetReferenceIdNotFoundForFilename;
function errAssetReferenceIdNotFoundForSetSource(assetReferenceId) {
    return {
        code: Errors.ASSET_NOT_FOUND,
        message: `Plugin error - Unable to set the source for unknown asset "${assetReferenceId}".`
    };
}
exports.errAssetReferenceIdNotFoundForSetSource = errAssetReferenceIdNotFoundForSetSource;
function errAssetSourceAlreadySet(asset) {
    return {
        code: Errors.ASSET_SOURCE_ALREADY_SET,
        message: `Plugin error - Unable to set the source for asset "${asset.name}", source already set.`
    };
}
exports.errAssetSourceAlreadySet = errAssetSourceAlreadySet;
function errAssetSourceMissingForSetSource(asset) {
    return {
        code: Errors.ASSET_SOURCE_MISSING,
        message: `Plugin error creating asset "${asset.name}", setAssetSource call without a source.`
    };
}
exports.errAssetSourceMissingForSetSource = errAssetSourceMissingForSetSource;
function errNoAssetSourceSet(asset) {
    return {
        code: Errors.ASSET_SOURCE_MISSING,
        message: `Plugin error creating asset "${asset.name}" - no asset source set.`
    };
}
exports.errNoAssetSourceSet = errNoAssetSourceSet;
function errBadLoader(id) {
    return {
        code: Errors.BAD_LOADER,
        message: `Error loading ${relativeId_1.default(id)}: plugin load hook should return a string, a { code, map } object, or nothing/null`
    };
}
exports.errBadLoader = errBadLoader;
function errChunkReferenceIdNotFoundForFilename(chunkReferenceId) {
    return {
        code: Errors.CHUNK_NOT_FOUND,
        message: `Plugin error - Unable to get file name for unknown chunk "${chunkReferenceId}".`
    };
}
exports.errChunkReferenceIdNotFoundForFilename = errChunkReferenceIdNotFoundForFilename;
function errInvalidAssetName(name) {
    return {
        code: Errors.INVALID_ASSET_NAME,
        message: `Plugin error creating asset, name "${name}" is not a plain (non relative or absolute URL) string name.`
    };
}
exports.errInvalidAssetName = errInvalidAssetName;
function errCannotAssignModuleToChunk(moduleId, assignToAlias, currentAlias) {
    return {
        code: Errors.INVALID_CHUNK,
        message: `Cannot assign ${relativeId_1.default(moduleId)} to the "${assignToAlias}" chunk as it is already in the "${currentAlias}" chunk.`
    };
}
exports.errCannotAssignModuleToChunk = errCannotAssignModuleToChunk;
function errInternalIdCannotBeExternal(source, importer) {
    return {
        code: Errors.INVALID_EXTERNAL_ID,
        message: `'${source}' is imported as an external by ${relativeId_1.default(importer)}, but is already an existing non-external module id.`
    };
}
exports.errInternalIdCannotBeExternal = errInternalIdCannotBeExternal;
function errInvalidOption(option, explanation) {
    return {
        code: Errors.INVALID_OPTION,
        message: `Invalid value for option "${option}" - ${explanation}.`
    };
}
exports.errInvalidOption = errInvalidOption;
function errInvalidRollupPhaseForAddWatchFile() {
    return {
        code: Errors.INVALID_ROLLUP_PHASE,
        message: `Cannot call addWatchFile after the build has finished.`
    };
}
exports.errInvalidRollupPhaseForAddWatchFile = errInvalidRollupPhaseForAddWatchFile;
function errInvalidRollupPhaseForEmitChunk() {
    return {
        code: Errors.INVALID_ROLLUP_PHASE,
        message: `Cannot call emitChunk after module loading has finished.`
    };
}
exports.errInvalidRollupPhaseForEmitChunk = errInvalidRollupPhaseForEmitChunk;
function errNamespaceConflict(name, reexportingModule, additionalExportAllModule) {
    return {
        code: Errors.NAMESPACE_CONFLICT,
        message: `Conflicting namespaces: ${relativeId_1.default(reexportingModule.id)} re-exports '${name}' from both ${relativeId_1.default(reexportingModule.exportsAll[name])} and ${relativeId_1.default(additionalExportAllModule.exportsAll[name])} (will be ignored)`,
        name,
        reexporter: reexportingModule.id,
        sources: [reexportingModule.exportsAll[name], additionalExportAllModule.exportsAll[name]]
    };
}
exports.errNamespaceConflict = errNamespaceConflict;
function errEntryCannotBeExternal(unresolvedId) {
    return {
        code: Errors.UNRESOLVED_ENTRY,
        message: `Entry module cannot be external (${relativeId_1.default(unresolvedId)}).`
    };
}
exports.errEntryCannotBeExternal = errEntryCannotBeExternal;
function errUnresolvedEntry(unresolvedId) {
    return {
        code: Errors.UNRESOLVED_ENTRY,
        message: `Could not resolve entry module (${relativeId_1.default(unresolvedId)}).`
    };
}
exports.errUnresolvedEntry = errUnresolvedEntry;
function errUnresolvedImport(source, importer) {
    return {
        code: Errors.UNRESOLVED_IMPORT,
        message: `Could not resolve '${source}' from ${relativeId_1.default(importer)}`
    };
}
exports.errUnresolvedImport = errUnresolvedImport;
function errUnresolvedImportTreatedAsExternal(source, importer) {
    return {
        code: Errors.UNRESOLVED_IMPORT,
        importer: relativeId_1.default(importer),
        message: `'${source}' is imported by ${relativeId_1.default(importer)}, but could not be resolved – treating it as an external dependency`,
        source,
        url: 'https://rollupjs.org/guide/en#warning-treating-module-as-external-dependency'
    };
}
exports.errUnresolvedImportTreatedAsExternal = errUnresolvedImportTreatedAsExternal;
